package eu.jonahbauer.raytracing.render;

import org.jetbrains.annotations.NotNull;

import java.util.Arrays;
import java.util.Objects;
import java.util.stream.Stream;

public final class Image {
    private final int width;
    private final int height;

    private final Color[][] data;

    public Image(int width, int height) {
        this.width = width;
        this.height = height;

        if (width <= 0) throw new IllegalArgumentException("width must be positive");
        if (height <= 0) throw new IllegalArgumentException("height must be positive");

        this.data = new Color[height][width];
    }

    public int width() {
        return width;
    }

    public int height() {
        return height;
    }

    public @NotNull Color get(int x, int y) {
        Objects.checkIndex(x, width);
        Objects.checkIndex(y, height);
        return Objects.requireNonNullElse(this.data[y][x], Color.BLACK);
    }

    public @NotNull Stream<Color> pixels() {
        return Arrays.stream(data).flatMap(Arrays::stream).map(c -> Objects.requireNonNullElse(c, Color.BLACK));
    }

    public void set(int x, int y, @NotNull Color color) {
        Objects.checkIndex(x, width);
        Objects.checkIndex(y, height);
        this.data[y][x] = Objects.requireNonNull(color);
    }

    public void set(int x, int y, int red, int green, int blue) {
        set(x, y, new Color(red, green, blue));
    }

    public void set(int x, int y, double r, double g, double b) {
        set(x, y, new Color(r, g, b));
    }
}
