package eu.jonahbauer.raytracing.scene;

import eu.jonahbauer.raytracing.math.Range;
import eu.jonahbauer.raytracing.math.Ray;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Optional;

public record Scene(@NotNull List<@NotNull Hittable> objects) implements Hittable {

    public Scene {
        objects = List.copyOf(objects);
    }

    public Scene(@NotNull Hittable @NotNull ... objects) {
        this(List.of(objects));
    }

    public @NotNull Optional<HitResult> hit(@NotNull Ray ray, @NotNull Range range) {
        var result = (HitResult) null;
        for (var object : objects) {
            var r = object.hit(ray, range);
            if (r.isPresent() && range.surrounds(r.get().t())) {
                result = r.get();
                range = new Range(range.min(), result.t());
            }
        }
        return Optional.ofNullable(result);
    }
}
